// MgaLibOps.cpp : Implementation of Mga library operations
#include "stdafx.h"
#include "MgaFCO.h"
#include "MgaSet.h"
#include "MgaReference.h"
#include "MgaConnection.h"
#include "MgaLibOps.h" // by ZolMol
#include "MgaLibRefr.h"
#include "../interfaces/Parser.h"

HRESULT FCO::get_LibraryName(BSTR *libname) {
	COMTRY {
		CHECK_OUTSTRPAR(libname);
		CheckRead();
		if(self[ATTRID_PERMISSIONS] & LIBROOT_FLAG) {
			CComBSTR nn = self[ATTRID_NAME];
			*libname = nn.Detach();
		}
		else *libname = NULL;
	} COMCATCH(;);
}


void setlibpermall(CoreObj &c, long newmask) {
	ASSERT(newmask == LIBRARY_FLAG || newmask == 0);
	ITERATE_THROUGH(c[ATTRID_PARENT+ATTRID_COLLECTION]) {
		if( newmask == 0) // remove all library flags (LIBRARY_FLAG, LIBROOT_FLAG), preserve only INSTANCE
			ITER[ATTRID_PERMISSIONS] = ITER[ATTRID_PERMISSIONS] & INSTANCE_FLAG;
		else // if applying LIBRARY_FLAG, preserve the LIBROOT_FLAG if previously present
			ITER[ATTRID_PERMISSIONS] = (ITER[ATTRID_PERMISSIONS] & (INSTANCE_FLAG|LIBROOT_FLAG)) | newmask;
		if(ITER.IsContainer()) setlibpermall(ITER, newmask);
	}
}	

HRESULT FCO::put_LibraryName(BSTR libname) {
	COMTRY_IN_TRANSACTION {
		CHECK_INSTRPAR(libname);
		CheckWrite();
		if(CComBSTR(libname)) {
			self[ATTRID_PERMISSIONS] = LIBROOT_FLAG;
			self[ATTRID_NAME] = libname;
			setlibpermall(self, LIBRARY_FLAG);
			SelfMark(OBJEVENT_PROPERTIES);
		}
		else {
			self[ATTRID_PERMISSIONS] = 0;
			setlibpermall(self, 0);
		}
	} COMCATCH_IN_TRANSACTION(;);
}

/* *************************** Pointer Fixup ************************************** */
/*
by ZolMol
class PointerFixup
{
	coreobjpairhash identity_map;

	typedef struct
	{
		CoreObj target;
		CoreObj created;
		attrid_type attrid;
	} resolve_entry;

	std::vector<resolve_entry> resolve_entries;

public:
	inline void identify(const CoreObj &original, const CoreObj &created)
	{
		ASSERT( original != NULL && created != NULL );

		identity_map.insert(coreobjpairhash::value_type(original, created));
	}

	void resolve(const CoreObj &original, const CoreObj &created,
		attrid_type attrid)
	{
		if( LINKREF_ATTR(attrid) )
			return;

		CoreObj target = original[attrid]; 
		if( target == NULL )
			return;

		coreobjpairhash::iterator i = identity_map.find(target);
		if( i != identity_map.end() )
			created[attrid] = i->second;	// already created
		else								// resolve it later
		{
			resolve_entry entry;
			entry.target = target;
			entry.created = created;
			entry.attrid = attrid;
			resolve_entries.push_back(entry);
		}
	}

	inline void clear()
	{
		resolve_entries.clear();
		identity_map.clear();
	}

	void fixPointers()
	{
		std::vector<resolve_entry>::const_iterator i = resolve_entries.begin();
		while( i != resolve_entries.end() )
		{
			CoreObj& new_target = identity_map[i->target];
			ASSERT( new_target != NULL );

			(i->created)[i->attrid] = new_target;

			++i;
		}
		clear();
	}
};
*/
/* *************************** Attach ********************************************* */

void ObjTreeCopyFromExt(CMgaProject *mgaproject, const CoreObj &orig, CoreObj &nobj, PointerFixup &fixup) {
	metaid_type s;
	COMTHROW(mgaproject->dataproject->CreateObject(s = orig.GetMetaID(), &nobj.ComPtr()));

	if( s != DTID_REGNODE )		// speedup, no references to regnodes
		fixup.identify(orig, nobj);

	if(s >= DTID_MODEL && s <= DTID_FOLDER) {
		setcheck(mgaproject, nobj, CHK_NEW);
		CoreObjMark(nobj, OBJEVENT_CREATED);
	}

	CComPtr<ICoreAttributes> atts;
	COMTHROW(orig->get_Attributes(&atts));
	MGACOLL_ITERATE(ICoreAttribute, atts) {
			attrid_type ai;
			CComPtr<ICoreMetaAttribute> mattr;
			COMTHROW(MGACOLL_ITER->get_MetaAttribute(&mattr));
			COMTHROW(mattr->get_AttrID(&ai));
			if(ai < ATTRID_COLLECTION) {
				valtype_type mvt;
				COMTHROW(mattr->get_ValueType(&mvt));
				if(mvt != VALTYPE_POINTER) {
					if(ai == ATTRID_PERMISSIONS) {
						//WAS: nobj[ai] = orig[ai] & INSTANCE_FLAG | LIBRARY_FLAG;
						//this way we preserve nested libraries:
						nobj[ai] = orig[ai] & (INSTANCE_FLAG|LIBROOT_FLAG) | LIBRARY_FLAG;
					}
					else nobj[ai] = static_cast<CComVariant>(orig[ai]);
				}
				else {
					fixup.resolve(orig, nobj, ai);
				}
			}
			else {
				ai -= ATTRID_COLLECTION;
				if(LINKREF_ATTR(ai)) {
					CoreObjs collmembers = orig[ai + ATTRID_COLLECTION];
					ITERATE_THROUGH(collmembers) {
						CoreObj nchild;
						ObjTreeCopyFromExt(mgaproject, ITER, nchild, fixup);
						nchild[ai] = nobj;
					}
				}
			}

	} MGACOLL_ITERATE_END;
}

void CreateLibraryImage(CMgaProject *mgaproject, CComBSTR &connstr, CoreObj &libimgroot, bool p_tolerateOldMga, bool *p_ptrIsOldMga) {
	CComPtr<IMgaProject> p;
	PointerFixup fixup;
	try {
		COMTHROW(p.CoCreateInstance(CLSID_MgaProject));
		
		CComBSTR paradigmname;
		CComVariant paradigmGUID;
		COMTHROW(mgaproject->get_MetaName(&paradigmname));
		COMTHROW(mgaproject->get_MetaGUID(&paradigmGUID));

		// ask some info about the project to be attached
		bool upgraded = false; CComBSTR connstr_upgraded( "MGA=");
		long mga_ver( 0);
		{
			CComBSTR par_nm;
			CComBSTR par_vs;     // not used now
			CComVariant par_gd;  // not used now
			VARIANT_BOOL ro_mode;// not used now 
			COMTHROW(p->QueryProjectInfo( connstr, &mga_ver, &par_nm, &par_vs, &par_gd, &ro_mode));
			if( par_nm != paradigmname)
			{
				// must upgrade 'connstr' project into 'paradigmname' paradigm
				VARIANT_BOOL ro = VARIANT_TRUE;
				HRESULT hr = p->Open( connstr, &ro);
				if( FAILED( hr)) {
					Reporter( mgaproject).show( CComBSTR( "Could not open the to-be-library project."));
					return;
				}

				// Find TEMP directory
				TCHAR szTempPath[MAX_PATH];
				if (::GetTempPath(MAX_PATH, szTempPath) == 0) {
					Reporter( mgaproject).show( CComBSTR( "Could not find TEMP directory."));
					return;
				}

				// Create temporary filenames 
				TCHAR szTempXmeFileName[MAX_PATH];
				TCHAR szTempMgaFileName[MAX_PATH];
				if( ::GetTempFileName(szTempPath, _T("GME"), 0, szTempXmeFileName) == 0  // 0 means ERROR
				 || ::GetTempFileName(szTempPath, _T("GME"), 0, szTempMgaFileName) == 0) {
					ASSERT(("Unable to get temporary filename.", false));
					Reporter( mgaproject).show( CComBSTR( "Could not get temporary filename."));
					return;
				}

				// Dump the to-be-library into XML
				CComPtr<IMgaDumper> dumper;
				COMTHROW( dumper.CoCreateInstance(L"Mga.MgaDumper") );
				ASSERT( dumper != NULL );

				COMTHROW(dumper->put_FormatVersion(0));
				COMTHROW(dumper->DumpProject( p, PutInBstr( szTempXmeFileName)) );

				p->Close();

				// Create a new 'paradigmname' project
				connstr_upgraded.Append( szTempMgaFileName); // connection string prepared
				hr = p->CreateEx( PutInBstr( connstr_upgraded), PutInBstr( paradigmname), paradigmGUID);
				if( SUCCEEDED( hr)) {
					CComPtr<IMgaParser> parser;
					COMTHROW( parser.CoCreateInstance(L"Mga.MgaParser") );
					ASSERT( parser != NULL );

					COMTHROW(parser->ParseProject( p, PutInBstr( szTempXmeFileName)) );
					p->Close();
					upgraded = true;
				}
				else if(hr == E_MGA_PARADIGM_NOTREG || hr == E_MGA_PARADIGM_INVALID) {
					ASSERT(0); // how come? these are similar to mgaproject's data
				}
			}
		}

		*p_ptrIsOldMga = mga_ver < 2;

		// check version of the mga file to be attached
		if( mga_ver < 2)
		{
			if( p_tolerateOldMga)
			{
				MyCComBSTR msg( "Library copy is in old MGA format. Library Refresh feature ver.1 (old) can be used only!");
				msg.Append( "To benefit from the new Library Refresh ver.2 feature please open/save the library with GME, then reattach it to this project.");
				Reporter( mgaproject).show( msg);
			}
			else
			{
				MyCComBSTR msg( "Library is in old MGA format. To update please open it as a project, save it and only then can be attached!");
				Reporter( mgaproject).show( msg);
				
				COMTHROW(E_MGA_NOT_SUPPORTED);
			}
		}

		COMTHROW(p->OpenEx( upgraded?connstr_upgraded:connstr, paradigmname, paradigmGUID));

		COMTHROW(p->BeginTransaction(NULL, TRANSACTION_READ_ONLY));
		{
			CComPtr<IMgaFolder> libroot;
			COMTHROW(p->get_RootFolder(&libroot));
	
			ObjTreeCopyFromExt(mgaproject, CoreObj(libroot), libimgroot, fixup);
		}
		fixup.fixPointers();

		libimgroot[ATTRID_NAME] = connstr;
		libimgroot[ATTRID_PERMISSIONS] = LIBROOT_FLAG;

		COMTHROW(p->AbortTransaction());
		COMTHROW(p->Close());
	} catch(...) { 
		fixup.clear();
		if(p) {
			long st;
			p->get_ProjectStatus(&st);
			if(st & 8) p->AbortTransaction();
			if(st & 1) p->Close();
		};
		throw;
	}
}

HRESULT FCO::AttachLibrary(BSTR libname, IMgaFolder **f, long relid) {
	COMTRY_IN_TRANSACTION {
		CHECK_OUTPTRPARVALIDNULL(f);
			
		CoreObj libimgroot;
		bool accept_old_lib( false); // don't tolerate old format MGA files (attachment of such a library fails)
		bool is_old_lib( false);
		CreateLibraryImage(mgaproject, CComBSTR(libname), libimgroot, accept_old_lib, &is_old_lib);

		assignnewchild(libimgroot);

		docheck(mgaproject);

		if(f) ObjForCore(libimgroot)->getinterface(f);

		ObjTreeNotify(mgaproject, libimgroot);
		SelfMark(OBJEVENT_NEWCHILD);
	} COMCATCH_IN_TRANSACTION(;);
}

/* *************************** Refresh ********************************************* */

void steal(CoreObj &o, CoreObj &n, attrid_type ai) {
	ITERATE_THROUGH(o[ai + ATTRID_COLLECTION]) {
		ITER[ai] = n;
	}
}

void redo_derivs(CMgaProject *mgaproject, CoreObj &oldnode, CoreObj &newnode, bool deriv_non_primary) {
	bool libnode = (newnode[ATTRID_PERMISSIONS] & LIBRARY_FLAG) ? true : false;
	if(long(oldnode[ATTRID_META]) != long(newnode[ATTRID_META])) {
		COMTHROW(E_MGA_LIB_DIFF);
	}
	if(oldnode.IsFCO()) {
		CoreObjs ders = oldnode[ATTRID_DERIVED+ATTRID_COLLECTION];
		ITERATE_THROUGH(ders) {
			long perm = ITER[ATTRID_PERMISSIONS];
			if(perm & LIBRARY_FLAG) continue;

			if(!deriv_non_primary && ITER[ATTRID_RELID] >= RELIDSPACE) continue;  // not primary derived
// 
			CoreObj newder;
			coreobjpairhash crealist;
			ObjTreeDerive(mgaproject, newnode, newder, crealist, perm & INSTANCE_FLAG);  
			newder[ATTRID_ROLEMETA] = ITER[ATTRID_ROLEMETA];  // maybe 0
			newder[ATTRID_PARENT] = ITER[ATTRID_PARENT];
			newder[ATTRID_RELID] = ITER[ATTRID_RELID];
							
			ObjTreeReconnect(newder, crealist); // Reroute references
			redo_derivs(mgaproject, ITER, newder, true);  // RECURSION!
		}
	}
	if(oldnode.IsContainer()) {
		CoreObjs oldchildren = oldnode[ATTRID_PARENT+ATTRID_COLLECTION];
		Sort<long>(oldchildren, ATTRID_RELID);
		CoreObjs newchildren = newnode[ATTRID_PARENT+ATTRID_COLLECTION];
		Sort<long>(newchildren, ATTRID_RELID);
		std::vector<CoreObj> newvect;
		GetAll2<ICoreObjects>(newchildren, newvect);
		std::vector<CoreObj>::iterator iter = newvect.begin(), iter_end = newvect.end();
		if(iter != iter_end) {
			long newrelid = (*iter)[ATTRID_RELID];
			ITERATE_THROUGH(oldchildren) {
				long oldrelid = ITER[ATTRID_RELID];
				while(newrelid < oldrelid && iter != iter_end )
				{
					if( ++iter != iter_end )
						newrelid = (*iter)[ATTRID_RELID];
				}
				// no refresh for objects with 0 relID
				if(oldrelid != 0 && oldrelid == newrelid) {
					redo_derivs(mgaproject, ITER, *iter, false);
				}
				else if(oldrelid < RELIDSPACE && !libnode) {
					ITER[ATTRID_PARENT] = newnode;
				}
			}
		}
	}

	// PETER : reroute references from the model
	if (libnode && oldnode.IsFCO()) {
		// Sets ? COnnections ? Other relations ?
		ITERATE_THROUGH(oldnode[ATTRID_REFERENCE + ATTRID_COLLECTION]) {
			if (ITER[ATTRID_PERMISSIONS] & LIBRARY_FLAG) continue;
			ITER[ATTRID_REFERENCE] = newnode;
		}
	}
	// PETER END

	if (libnode) 
		return;

	ASSERT(newnode[ATTRID_RELID] == oldnode[ATTRID_RELID]);
	ASSERT(newnode[ATTRID_META] == oldnode[ATTRID_META]);

	newnode[ATTRID_NAME] = oldnode[ATTRID_NAME];
	if(oldnode.IsContainer()) {
		newnode[ATTRID_LASTRELID] = oldnode[ATTRID_LASTRELID];
	}
	steal(oldnode, newnode, ATTRID_REGNOWNER);
	steal(oldnode, newnode, ATTRID_CONSTROWNER);

	if(oldnode.IsFCO()) {
		ASSERT(newnode[ATTRID_ROLEMETA] == oldnode[ATTRID_ROLEMETA]);
		steal(oldnode, newnode, ATTRID_ATTRPARENT);
		steal(oldnode, newnode, ATTRID_XREF);
		steal(oldnode, newnode, ATTRID_REFERENCE);
		switch(oldnode.GetMetaID()) {
		case DTID_REFERENCE:
			steal(oldnode, newnode, ATTRID_SEGREF);
			if(!CoreObj(oldnode[ATTRID_MASTEROBJ])) {
				std::vector<CoreObj> peer(1);
				peer[0] = oldnode[ATTRID_REFERENCE];   // set the old val, it will
													   // be updated through xref
				newnode[ATTRID_MASTEROBJ] = NULLCOREOBJ;
				putreftask(false).DoWithDeriveds(newnode, &peer);
			}
			break;
		case DTID_CONNECTION:
			{
				ITERATE_THROUGH(oldnode[ATTRID_CONNROLE + ATTRID_COLLECTION]) {
					if(!CoreObj(ITER[ATTRID_MASTEROBJ])) {
						CoreObj rr;
						if(ObjForCore(newnode)->findroleobj(CComBSTR(ITER[ATTRID_NAME]), rr)) {
							RemoveConnPTask().DoWithDeriveds(rr);
						}
// todo: propagate
						ITER[ATTRID_CONNROLE] = newnode;
					}
				}
			}
			break;
		case DTID_SET:
			{
				ITERATE_THROUGH(oldnode[ATTRID_SETMEMBER + ATTRID_COLLECTION]) {
					if(!CoreObj(ITER[ATTRID_MASTEROBJ])) {
// todo: delete duplicate setnodes
// todo: propagate
						ITER[ATTRID_SETMEMBER] = newnode;
					}
				}
			}
			break;
		}
	}
}


HRESULT FCO::RefreshLibrary(BSTR libname) {
	COMTRY_IN_TRANSACTION {
		if( !Identifier::isRefreshableLibRoot( self))
		{
			MyCComBSTR msg( "Inner library can't be refreshed in this project!");
			Reporter( mgaproject).show( msg);
		} 
		else 
		{
			CheckWrite();
			if(!(self[ATTRID_PERMISSIONS] & LIBROOT_FLAG)) {
				COMTHROW(E_MGA_OP_REFUSED);
			}

			CComBSTR ln(libname);
			if(!ln) COMTHROW(get_LibraryName(&ln));
			CoreObj libimgroot;
			bool accept_old_lib( true); // refreshment of an old format MGA file is allowed
			bool is_old_lib_copy( false);
			CreateLibraryImage(mgaproject, ln, libimgroot, accept_old_lib, &is_old_lib_copy);
	 
			libimgroot[ATTRID_PARENT] = self[ATTRID_PARENT];
			libimgroot[ATTRID_RELID] = self[ATTRID_RELID];

			if( is_old_lib_copy) // do a plain old refresh
			{
				MyCComBSTR msg( "Old version of Library refresh started [");
				msg.AppendBSTR( libname); msg.Append( "] --");
				Reporter( mgaproject).show( msg, false);

				redo_derivs(mgaproject, self, libimgroot, false);
				inDeleteObject();
				docheck(mgaproject);
			}
			else // do a uid based refresh
			{
				MyCComBSTR msg( "---------------------Library refresh v2 started [");
				msg.AppendBSTR( libname); msg.Append( "] --");
				Reporter( mgaproject).show( msg, false);
				
				RefreshManager rm( mgaproject, self, libimgroot);
				ASSERT( Identifier::isLibRoot( self));

				rm.collectDependencies( self);
				rm.cutRelations( self);
				//redo_derivs(mgaproject, self, libimgroot, false);
				rm.restoreDependencies();
				rm.clearDepStorage();

				inDeleteObject();

				try {
					docheck(mgaproject);

					MyCComBSTR msg;
					msg.Append( "----------------------Library refresh done--------");
					rm.getNumOfErrors( msg);
					msg.Append( "--");
					Reporter( mgaproject).show( msg, false);

				} catch(hresult_exception& ) {
					MyCComBSTR msg( "Check failed after refresh!");
					Reporter( mgaproject).show( msg);
					throw;
				}
			}
		} // else
	} COMCATCH_IN_TRANSACTION( Reporter(mgaproject).show( MyCComBSTR( "----------------------Library refresh failed----------------------")));
}

